Imports System.IO
Imports System.Security
Imports System.Security.Cryptography

Public Class Form1
    Private Sub Form1_Load() Handles MyBase.Load
        Dim file_path As String = My.Computer.FileSystem.CombinePath(Application.StartupPath, "..\..")
        txtPlaintextFile.Text = file_path & "\plaintext.txt"
        txtPlaintextFile.Select(txtPlaintextFile.Text.Length, 0)
        txtCyphertextFile.Text = file_path & "\cyphertext.dat"
        txtCyphertextFile.Select(txtCyphertextFile.Text.Length, 0)
        txtDecypheredFile.Text = file_path & "\decyphered.txt"
        txtDecypheredFile.Select(txtDecypheredFile.Text.Length, 0)

        ' Display the original file.
        txtPlaintext.Text = File.ReadAllText(txtPlaintextFile.Text)
    End Sub

    ' Encrypt or decrypt a file, saving the results 
    ' in another file.
    Private Sub CryptFile(ByVal password As String, ByVal in_file As String, ByVal out_file As String, ByVal encrypt As Boolean)
        ' Create input and output file streams.
        Using in_stream As New FileStream(in_file, FileMode.Open, FileAccess.Read)
            Using out_stream As New FileStream(out_file, FileMode.Create, FileAccess.Write)

                ' Make a triple DES service provider.
                Dim des_provider As New TripleDESCryptoServiceProvider()

                ' Find a valid key size for this provider.
                Dim key_size_bits As Integer = 0
                For i As Integer = 1024 To 1 Step -1
                    If des_provider.ValidKeySize(i) Then
                        key_size_bits = i
                        Exit For
                    End If
                Next i
                Debug.Assert(key_size_bits > 0)

                ' Get the block size for this provider.
                Dim block_size_bits As Integer = des_provider.BlockSize

                ' Generate the key and initialization vector.
                Dim key As Byte() = Nothing
                Dim iv As Byte() = Nothing
                Dim salt As Byte() = {&H0, &H0, &H1, &H2, &H3, &H4, &H5, &H6, &HF1, &HF0, &HEE, &H21, &H22, &H45}
                MakeKeyAndIV(password, salt, key_size_bits, block_size_bits, key, iv)

                ' Make the encryptor or decryptor.
                Dim crypto_transform As ICryptoTransform
                If encrypt Then
                    crypto_transform = des_provider.CreateEncryptor(key, iv)
                Else
                    crypto_transform = des_provider.CreateDecryptor(key, iv)
                End If

                ' Attach a crypto stream to the output stream.
                ' Closing crypto_stream sometimes throws an
                ' exception if the decryption didn't work
                ' (e.g. if we use the wrong password).
                Try
                    Using crypto_stream As New CryptoStream(out_stream, crypto_transform, CryptoStreamMode.Write)
                        ' Encrypt or decrypt the file.
                        Const BLOCK_SIZE As Integer = 1024
                        Dim buffer(BLOCK_SIZE) As Byte
                        Dim bytes_read As Integer
                        Do
                            ' Read some bytes.
                            bytes_read = in_stream.Read(buffer, 0, BLOCK_SIZE)
                            If bytes_read = 0 Then Exit Do

                            ' Write the bytes into the CryptoStream.
                            crypto_stream.Write(buffer, 0, bytes_read)
                        Loop

                        ' Close the streams.
                        crypto_stream.Flush()
                        crypto_stream.Close()
                    End Using ' crypto_stream 
                Catch ex As Exception
                End Try

                crypto_transform.Dispose()
                in_stream.Close()
                out_stream.Close()
            End Using ' out_stream
        End Using ' in_stream
    End Sub

    ' Use the password to generate key bytes.
    Private Sub MakeKeyAndIV(ByVal password As String, ByVal salt() As Byte, ByVal key_size_bits As Integer, ByVal block_size_bits As Integer, ByRef key As Byte(), ByRef iv As Byte())
        Dim derive_bytes As New Rfc2898DeriveBytes( _
            password, salt, 1000)

        key = derive_bytes.GetBytes(key_size_bits \ 8)
        iv = derive_bytes.GetBytes(block_size_bits \ 8)

        ' Debugging:
#Const DEBUG_KEY = False
#If DEBUG_KEY Then
            Dim txt As String
            txt = ""
            For i As Integer = 0 To salt.Length - 1
                txt &= salt(i).ToString() & " "
            Next i
            Debug.WriteLine("Salt: " & txt)

            txt = ""
            For i As Integer = 0 To key.Length - 1
                txt &= key(i).ToString() & " "
            Next i
            Debug.WriteLine("Key:  " & txt)

            txt = ""
            For i As Integer = 0 To iv.Length - 1
                txt &= iv(i).ToString() & " "
            Next i
            Debug.WriteLine("IV:   " & txt)
            Debug.WriteLine("******")
#End If ' DEBUG_BYTES
    End Sub

    ' Encrypt the plaintext file.
    ' Note the values
    Private Sub btnEncrypt_Click() Handles btnEncrypt.Click
        ' Encrypt the file.
        CryptFile(txtPassword.Text, txtPlaintextFile.Text, txtCyphertextFile.Text, True)

        ' Display the result.
        txtCyphertext.Text = File.ReadAllText(txtCyphertextFile.Text)
    End Sub

    ' Decrypt the cyphertext file.
    Private Sub btnDecrypt_Click() Handles btnDecrypt.Click
        ' Encrypt the file.
        CryptFile(txtPassword.Text, txtCyphertextFile.Text, txtDecypheredFile.Text, False)

        ' Display the result.
        txtDecyphered.Text = File.ReadAllText(txtDecypheredFile.Text)
    End Sub
End Class
